/************************************************** 
  Differential input voltmeter - written by rAYMOND THE sANE  2020

  Audibly announces voltage via DFPlayer Mini module. 

  Uses differential inputs to measure up to 20 volts DC and 14 volts AC.
  Automatically detects AC and polarity
  Input resistance approx 1Meg Ohms if used with accompanying circuitry
  Resolution deliberately held to 0.1 volts - who needs better than that on a workbench anyway?
  Voltage is measured at 2 second intervals
  
  Works with schematic and PCB "talkidiffvmbrdmk1" plus associated mp3 files
 ****************************************************/
#include <DFPlayer_Mini_Mp3.h>
//#include <IRremote.h>  //schematic and pcb have provision for IR remote components - not used in this instance
 
#define reads 10  // sets number of measurements taken per cycle
#define pin1 A2  //ADC pin1
#define pin2 A7  //ADC pin2
#define playing 10  //DFplayer state detect pin
#define timer 1000 //set time between read cycles in milliseconds ... recommended minimum 1000
double vmax, v, VR[reads], VR2[reads];
int i, j, vt, dec;
bool neg, pos;
String polarity;

int vol=20; // set maximum volume ... range 0-30
double divide=18.4;  //set this to actual resistor divider ratio.  Adjust against known voltage until correct.

void speakvoltage(){  // speak the voltage and polarity
  
  mp3_set_volume(vol);
  delay(10);
//  Serial.print(" playing ");
//  Serial.println(int(vmax));

if (int(vmax) <= 20){
  if (int(vmax) > 0){
  mp3_play(int(vmax));  //speak volts
  }
  else if  
  (int(vmax) ==0){mp3_play(0);}  //speak zero volts
  
  delay(1000);   // delay to allow speech file to end - used frequently
  mp3_play(104); //speak point
  delay(600);
  
  dec =(vmax*10)+.5;  //round hundredths up 
  dec = dec % 10;  // calculate tenths of volts
  
  if (dec > 0){
  mp3_play(dec);  // speak tenths of volts
  }
  else if 
  (dec == 0){mp3_play(0);}  //speak zero decimal point volts
    
  delay(800);
 mp3_play(107);  //volts
  delay(600);

  
switch (vt) { //speak polarity
  case 1:
  Serial.print("positive");
  mp3_play(101);  //speak positive 
  delay(1000);
  break;
  case 2:
  Serial.print("negative");
  mp3_play(102); //speak negative
  delay(1000);
  break;
  case 3:
  Serial.print("AC");
  mp3_play(103); //speak ac
  delay(1000);
  break;  
  case 4:
  Serial.print("none");
  mp3_play(105);  //speak voltmeter ready
  delay(1200);
  break;
}
}


else {
    mp3_play(106); //speak overload
    delay(800);
  }
  
}

void setup() {
  Serial.begin(9600);
  Serial.println("   diffvmtalkermk10  October 2020   "); //I do this on all programs so that the MCU will announce what software it contains. 
  pinMode(pin1, INPUT);
  pinMode(pin2, INPUT);
  pinMode(playing, INPUT);
  mp3_set_serial (Serial);  // set DFPlayer serial speed
  delay(10);  // delay to allow DFPlayer to operate and set pin states. - used frequently
  mp3_stop();  //ensure DFPlayer not playing
  delay(10);
  mp3_set_volume (vol);  // set DFPlayer volume
  delay(10);
  mp3_set_EQ (5);  // set DFPlayer equalizer - 5 is bass.
  delay(10);
  mp3_play(105);  // speak voltmeter ready
  delay(1200);
  }

void loop(){ 
  vmax=0;  //set voltage to zero
  pos = 0;  //set flags to initial setting
  neg = 0;
  polarity = "none";
  
if (digitalRead(playing == HIGH)){  //proceed only if the DFPlayer is not playing
  
for (int i=0; i<reads; i++)  //read the voltage "reads" number of times - store in array
    {
    VR[i] = analogRead(pin1);
    VR2[i] = analogRead(pin2);
         delay(2);
    }

 
  for (int j=0; j <reads; j++)  //read the voltage array 
        {
        v = (VR[j]-VR2[j]) /divide;  //get analog readings and divide by resistor ratio.
       if (v > 0.1) {pos = 1;}  //set flag for positive readings - allow 0.1 volts for noise
       if (v < -0.1) {neg = 1;}  //set flag for negative readings - allow 0.1 volts for noise
     
       Serial.print(" \t");
       Serial.print(v);
       if (abs(v) > vmax) {vmax = abs(v);}  //get maximum peak voltage - "abs" removes polarity = software rectifier
         }
         
        if ((pos == 1) && (neg == 1)) {polarity = "AC"; vt = 3;} //Determine if AC or DC and DC polarity
        else if ((pos == 1) && (neg == 0)) {polarity = "positive";vt = 1;}
        else if ((pos == 0) && (neg == 1)) {polarity = "negative";vt = 2;}
        else {polarity = "none";vt =4;}

        if (vt == 3) {vmax = (vmax *.71);} //convert maximum peak voltage to approximate RMS
        Serial.print(" vmax  \t");
        Serial.print(vmax);
        Serial.print(" Polarity \t");
        Serial.println(polarity);  
      

        
if (vmax >= .1) {speakvoltage();}  //do not speak anything unless voltage is greater than 0.1 volts
}
  delay(timer);  //wait between read cycles
}
